import { Utilities } from '@/util/Utilities';
import {
    Modal, ModalService, Button, TextInput, Text, Checkbox, Textarea, Date as ZdDate, Form, Time, LoadingService, FileInput,
} from '@zeedhi/common';
import {
    Metadata, Http, IDictionary, Singleton, IEventParam, Router,
} from '@zeedhi/core';
import {
    IterableComponentRender,
} from '@zeedhi/teknisa-components-common';
import FileUpload from './FileUpload';

@Singleton
export class ApontamentoController {
    private modalEntry!: Modal;

    private modalExit!: Modal;

    private modalValidate!: Modal;

    private NRAPONTAMENTO!: number;

    private LASTDATEENTRY!: string;

    public async createModal() {
        this.modalEntry = await ModalService.createFromJsonPost('', { jsonPath: 'fm_00021_000_apontamento/modal/fm_00021_000_registrar_entrada' });
        this.modalExit = await ModalService.createFromJsonPost('', { jsonPath: 'fm_00021_000_apontamento/modal/fm_00021_000_registrar_saida' });
        this.modalValidate = await ModalService.createFromJsonPost('', { jsonPath: 'fm_00021_000_apontamento/modal/fm_00021_000_validar_saida' });
    }

    public destroyModal() {
        this.modalValidate?.destroy();
        this.modalEntry?.destroy();
        this.modalExit?.destroy();
    }

    public openModalEntry() {
        this.modalEntry.show();
    }

    public openModalExit() {
        this.modalExit.show();
    }

    public openModalValidate() {
        this.modalValidate.show();
    }

    public closeModalValidate() {
        this.modalValidate.hide();
    }

    public closeModalEntry() {
        this.modalEntry.hide();
    }

    public closeModalExit() {
        this.modalExit.hide();
    }

    public openModalAppointment({ component }: { component: Button }) {
        if (component.label === 'Entrada') {
            this.openModalEntry();
        } else {
            this.openModalExit();
        }
    }

    public setButtonRegister() {
        Http.get('/getUltimoRegistro').then((response: IDictionary) => {
            response.data.forEach((data: IDictionary) => {
                if (data.IDTIPORESGISTRO === 'E') {
                    Metadata.getInstance<Button>('homeAppointment').label = 'Saída';
                } else {
                    Metadata.getInstance<Button>('homeAppointment').label = 'Entrada';
                }
                this.NRAPONTAMENTO = data.NRAPONTAMENTO;
                this.LASTDATEENTRY = data.DTHRENTRADA;
            });
        });
    }

    public getCurrentDate() {
        const date = new Date();

        const hour = date.getHours();
        const min = date.getMinutes();

        const dateNow = `${date.toLocaleDateString()} ${hour}:${min}`;
        return dateNow;
    }

    public setCurrentDate() {
        const timeApp = this.getCurrentDate().split(' ')[1];
        const shortDate = this.getCurrentDate().split(' ')[0];
        Metadata.getInstance<Time>('input-time-appointment').value = timeApp;
        Metadata.getInstance<ZdDate>('date-appointment').value = shortDate;
        Metadata.getInstance<Text>('text-appointment-date-now').text = `Apontamento do dia: ${shortDate}`;
    }

    public async registerAbsence() {
        const dateOfAbsence = Metadata.getInstance<ZdDate>('selectDate').value;
        const justify = Metadata.getInstance<Textarea>('input-justify-appointment').value;
        const form = Metadata.getInstance<Form>('form-appointment-absence');
        const imagensConvertidas = await FileUpload.converterImagens('file-input-falta');

        if (Utilities.validateForm(form)) {
            Http.post('/registerAbsence', {
                DTAPONTAMENTO: dateOfAbsence,
                IDFALTA: 'S',
                DSOBSREGISTRO: justify,
                IMAGENS: imagensConvertidas,
            }).then(() => {
                Utilities.showSuccessMessage();
                this.limparCamposRegistrarFalta();
            }).catch((error: any) => {
                Utilities.showErrorMessage(error);
            }).finally(() => {
                LoadingService.hide();
            });
        }
    }

    public limparCamposRegistrarFalta() {
        Metadata.getInstance<ZdDate>('selectDate').value = null;
        Metadata.getInstance<Textarea>('input-justify-appointment').value = null;
        Metadata.getInstance<FileInput>('file-input-falta').value = null;
    }

    public async registerByAppointment() {
        const dateAppointment = Metadata.getInstance<TextInput>('date-appointment').value;
        const timeAppointment = Metadata.getInstance<Time>('input-time-appointment').value;
        const dateAndTimeApp = `${dateAppointment} ${timeAppointment}`;
        const observation = Metadata.getInstance<Textarea>('input-observation-appointment').value;
        Metadata.getInstance<IterableComponentRender>('iCardsAppointment').datasource.data = [];
        const imagensConvertidas = await FileUpload.converterImagens('file-input-apontamento');

        Http.get('/getUltimoRegistro').then((response: IDictionary) => {
            response.data.forEach((data: IDictionary) => {
                if (data.IDTIPORESGISTRO === 'E') {
                    if (dateAppointment !== this.LASTDATEENTRY) {
                        Promise.all([this.openModalValidate()]).then(() => {
                            this.setDateModal();
                        });
                    } else {
                        const almoco = (Metadata.getInstance<Checkbox>('checkbox-lunch').value) ? 'S' : 'N';
                        this.registrarApontamento(timeAppointment, null, dateAppointment, observation, 'S', dateAndTimeApp, almoco, imagensConvertidas);
                    }
                } else {
                    this.registrarApontamento(timeAppointment, dateAndTimeApp, dateAppointment, observation, 'E', null, 'N', imagensConvertidas);
                }
            });
        });
    }

    public registrarApontamento(hora: any, dataEntrada: any, dataApontamento: any, observacao: string, tipoRegistro: string, dataSaida: any, almoço: string, imagens: any) {
        const cards = Metadata.getInstance<IterableComponentRender>('iCardsAppointment');
        const checkbox = Metadata.getInstance<Checkbox>('checkbox-lunch');

        LoadingService.show();
        Http.post('/setNewRegister', {
            CURRENTTIME: hora,
            DTHRENTRADA: dataEntrada,
            DTAPONTAMENTO: dataApontamento,
            DSOBSREGISTRO: observacao,
            IDTIPORESGISTRO: tipoRegistro,
            DTHRSAIDA: dataSaida,
            IDALMOCO: almoço,
            IMAGENS: imagens,
        }).then(() => {
            Utilities.showSuccessMessage();
            cards.datasource.get();
            this.setButtonRegister();
            checkbox.value = null;
            Metadata.getInstance<FileInput>('file-input-apontamento').value = null;
            this.desabilitarCheckboxAlmoco({ component: checkbox });
        }).finally(() => {
            LoadingService.hide();
        });
    }

    public abrirApontamento({ component }: IEventParam<Button>) {
        Router.replace('/fm_00021_000_registrar_apontamento');
        if (component.name === 'abrir-apontamento-saida') {
            this.closeModalExit();
        } else if (component.name === 'abrir-apontamento-entrada') {
            this.closeModalEntry();
        }
    }

    public registerExitLastDate(currentDate: string) {
        const currentTime = this.getCurrentDate().split(' ')[1];

        LoadingService.show();
        Http.post('/setLastAppointment', {
            CURRENTTIME: currentTime,
            DTHRENTRADA: null,
            DTAPONTAMENTO: this.LASTDATEENTRY,
            NRAPONTAMENTO: this.NRAPONTAMENTO,
            DSOBSREGISTRO: null,
            IDTIPORESGISTRO: 'S',
            DTHRSAIDA: currentDate,
            IDALMOCO: 'N',
        }).then(() => {
            Utilities.showSuccessMessage();
            this.closeModalExit();
        }).catch((error: any) => {
            Utilities.showErrorMessage(error);
        }).finally(() => {
            LoadingService.hide();
            this.closeModalValidate();
        });
    }

    public registerExitDateToday(dateExit: string) {
        const currentTime = this.getCurrentDate().split(' ')[1];
        const cards = Metadata.getInstance<IterableComponentRender>('iCardsAppointment');

        LoadingService.show();
        Http.post('/setNewRegister', {
            CURRENTTIME: currentTime,
            DTHRENTRADA: null,
            DTAPONTAMENTO: dateExit,
            DSOBSREGISTRO: null,
            IDTIPORESGISTRO: 'S',
            DTHRSAIDA: dateExit,
            IDALMOCO: 'N',
        }).then(() => {
            Utilities.showSuccessMessage();
            cards.datasource.get();
            this.setButtonRegister();
        }).catch((error) => {
            Utilities.showErrorMessage(error);
        }).finally(() => {
            LoadingService.hide();
            this.closeModalExit();
            this.closeModalEntry();
            this.closeModalValidate();
        });
    }

    public setDateModal() {
        const buttonLastDate = Metadata.getInstance<Button>('button-register-lastDate');
        buttonLastDate.label = `Registrar: ${this.LASTDATEENTRY}`;
    }

    public registerExitByValidateModal({ component }: { component: Button }) {
        const currentDate = this.getCurrentDate();

        if (component.name === 'button-register-today') {
            this.registerExitDateToday(currentDate);
        } else if (component.name === 'button-register-lastDate') {
            this.registerExitLastDate(this.LASTDATEENTRY);
        }
    }

    public async registerEntryOrExitByModal({ component }: { component: Button }) {
        const currentDate = this.getCurrentDate();
        const currentTime = this.getCurrentDate().split(' ')[1];
        const cards = Metadata.getInstance<IterableComponentRender>('iCardsAppointment');

        if (String(currentDate.split(' ')[0]) !== this.LASTDATEENTRY && component.name !== 'button-register-entry') {
            Promise.all([this.openModalValidate()]).then(() => {
                this.setDateModal();
            });
        } else {
            Http.post('/setNewRegister', {
                CURRENTTIME: currentTime,
                DTAPONTAMENTO: null,
                DTHRENTRADA: (component.name === 'button-register-entry') ? currentDate : null,
                DSOBSREGISTRO: null,
                IDTIPORESGISTRO: (component.name === 'button-register-exit' || component.name === 'button-register-lunch') ? 'S' : 'E',
                DTHRSAIDA: (component.name === 'button-register-exit' || component.name === 'button-register-lunch') ? currentDate : null,
                IDALMOCO: (component.name === 'button-register-lunch') ? 'S' : 'N',
            }).then(() => {
                Utilities.showSuccessMessage();
                cards.datasource.get();
                this.setButtonRegister();
            }).catch((error) => {
                Utilities.showErrorMessage(error);
            }).finally(() => {
                LoadingService.hide();
                this.closeModalExit();
                this.closeModalEntry();
            });
        }
    }

    public desabilitarCheckboxAlmoco({ component }: any) {
        Http.get('/optionLunch').then((response: IDictionary) => {
            response.data.forEach((dados: any) => {
                if (dados.IDTIPORESGISTRO !== 'E' || dados.QTDALMOCO === 1) {
                    component.disabled = true;
                }
            });
        }).catch((error) => {
            Utilities.showErrorMessage(error);
        }).finally(() => {
            LoadingService.hide();
        });
    }

    public desabilitarAlmoçoModal({ component }: IEventParam<Button>) {
        Http.get('/optionLunch').then((response: IDictionary) => {
            response.data.forEach((dados: any) => {
                if (dados.QTDALMOCO === 1) {
                    component.disabled = true;
                }
            });
        }).catch((error) => {
            Utilities.showErrorMessage(error);
        }).finally(() => {
            LoadingService.hide();
        });
    }
}
