import { Utilities } from '@/util/Utilities';
import {
    DateRange, Form, LoadingService, Select, TextInput, Text,
} from '@zeedhi/common';
import prodEnvironment from '@/config/prodEnvironment.json';
import devEnvironment from '@/config/devEnvironment.json';
import {
    Config, Http, Loader, Metadata,
} from '@zeedhi/core';
import {
    clearStorage, getStoredData, reload,
} from '@zeedhi/zeedhi-miragejs';

export class ParametrizacaoController {
    private tarefaOffline = Loader.getInstance('Fm00001TarefasControllerOffline');

    public tipoCarregamento: String = '';

    public text = '';

    public visibilidadeTab = false;

    public qtdDias: any;

    public habilitaOffline: any;

    public tpOffline: any;

    public dtFinalInicial: any;

    public syncDadosBackEnd() {
        this.tarefaOffline.prepararDadosIndexDb();
    }

    public setaValorDias() {
        const xDias = Number(Metadata.getInstance<TextInput>('qtdDiasParam').value);
        const diaAtual = new Date();
        const diaFinal = new Date(diaAtual.getFullYear(), diaAtual.getMonth(), diaAtual.getDate() + xDias);
        this.text = `Período escolhido: ${diaAtual.getDate()}/${diaAtual.getMonth() + 1}/${diaAtual.getFullYear()} até ${diaFinal.getDate()}/${diaFinal.getMonth() + 1}/${diaFinal.getFullYear()}`;
    }

    public changeTipoCarregamento({ component }: any) {
        const xDias = Metadata.getInstance<TextInput>('qtdDiasParam');
        const personalizado = Metadata.getInstance<DateRange>('dataParam');
        const exibirDatas = Metadata.getInstance<Text>('textAreaDias');
        if (component.value === '1') {
            xDias.isVisible = true;
            personalizado.isVisible = false;
            personalizado.value = null;
            exibirDatas.isVisible = true;
        } else {
            personalizado.isVisible = true;
            xDias.isVisible = false;
            xDias.value = null;
            exibirDatas.isVisible = false;
        }
        this.tipoCarregamento = component.value;
    }

    public salvarPrametrizacoes() {
        const xDias = Metadata.getInstance<TextInput>('qtdDiasParam').value;
        const personalizado = Metadata.getInstance<DateRange>('dataParam').value;
        const modoOffline = Metadata.getInstance<Select>('selectSimpleResult').value;
        const form = Metadata.getInstance<Form>('form-filtros');
        if (Utilities.validateForm(form)) {
            LoadingService.show();
            Http.post('/parametrizacaoOfflineApp', {
                XDIAS: xDias,
                DATARANGE: personalizado,
                TIPOCARREGAMENTO: this.tipoCarregamento,
                USAOFFLINE: modoOffline,
            }).then(() => {
                Utilities.showSuccessMessage();
            }).catch((error: any) => {
                Utilities.showErrorMessage(error);
            }).finally(() => {
                LoadingService.hide();
            });
        }
    }

    public buscarInformacoesParam() {
        LoadingService.show();
        Http.get('/getParamOffline').then((response) => {
            this.qtdDias = response.data[0].QTDIASCARREGAOFFLINE;
            this.habilitaOffline = response.data[0].IDHABILITAOFFLINE;
            this.tpOffline = response.data[0].IDTIPOCARREGAOFFLINE;
            this.dtFinalInicial = [response.data[0].DTINICAL];
            this.dtFinalInicial.push([response.data[0].DTFINAL]);
            if (response.data[1].length > 14) {
                this.visibilidadeTab = true;
            } else {
                this.visibilidadeTab = false;
            }
        }).catch((error: any) => {
            Utilities.showErrorMessage(error);
        }).finally(() => {
            LoadingService.hide();
        });
    }

    public configurarParam() {
        Metadata.getInstance<TextInput>('qtdDiasParam').value = this.qtdDias;
        Metadata.getInstance<Select>('selectSimpleResult').value = this.habilitaOffline;
        Metadata.getInstance<Select>('defineEscolhaData').value = this.tpOffline;
        Metadata.getInstance<DateRange>('dataParam').value = this.dtFinalInicial;
    }

    public async teste() {
        await clearStorage();
    }

    public async testeReload() {
        await reload();
    }

    // Função responsável por sincronizar todos os dados do app offline.
    public async baixarDadosIndexDB() {
        LoadingService.show();
        const tarefas = await getStoredData('getMinhasTarefas');
        const itensTarefas = await getStoredData('itensTarefasAlteradosOffline');
        const fotosAnexadas = await getStoredData('anexarFotosOffline');
        const fotosExcluidas = await getStoredData('excluirFotoOffline');
        Http.post('/syncDadosMirage', {
            TAREFAS: tarefas,
            ITENS: itensTarefas,
            ANEXARFOTO: fotosAnexadas,
            REMOVERFOTO: fotosExcluidas,
        }).then(async () => {
            await clearStorage();
            Utilities.showSuccessMessage();
        }).catch((error: any) => {
            Utilities.showErrorMessage(error);
        }).finally(async () => {
            await reload();
            LoadingService.hide();
        });
    }

    public mudarRotaMirage() {
        if (window.localStorage.getItem('MIRAGEJS') === 'S') {
            window.localStorage.setItem('MIRAGEJS', 'N');
            const env = process.env.NODE_ENV === 'production' ? prodEnvironment : devEnvironment;
            Config.endPoint = env.endPoint;
            Http.setBaseURL(Config.endPoint);
        } else {
            window.localStorage.setItem('MIRAGEJS', 'S');
            Config.endPoint = 'http://teste/mirage';
            Http.setBaseURL(Config.endPoint);
        }
    }
}
